//Most of what's here drawn from the excellent
//http://www.elec-cafe.com/temperature-sensor-on-the-web-with-esp8266-and-ds18b20/

//Changed use of what's here to avoid the "delay(1000)" required by the original's
//approach.

//Some modifications have been made by TKB, 1/17

void OneWireCheckChipAndInitVars(byte bWhich)
{
//N.B.: This includes provision for all of the chips supported
//by the original version. In some of my before-5 Jan 17 adapations
//of elec-cafe's code, I cut that provision back.
//
//The two alternate blocks done crudely. Could probably be made better.
if (bWhich==0)
{
if ( !ds0.search(addr0))
{
  Serial.println("No more addresses on ds0.");
  Serial.println();
  ds0.reset_search();
  delay(250);
  return;
}

//Report the "ID" of the chip found...
Serial.print("ROM =");
for ( i = 0; i < 8; i++)
{
  Serial.write(' ');
  Serial.print(addr0[i], HEX);
}

if (OneWire::crc8(addr0, 7) != addr0[7])
{
  Serial.println("CRC is not valid for ds0!");
  return;
}
Serial.println();

// the first ROM byte indicates which chip
Serial.print("Chip on ds0: ");
switch (addr0[0])
{
case 0x10:
Serial.println("  Chip = DS18S20");  // or old DS1820
type_s0 = 1;
break;
case 0x28:
Serial.println("  Chip = DS18B20");
type_s0 = 0;
break;
case 0x22:
Serial.println("  Chip = DS1822");
type_s0 = 0;
break;
default:
Serial.println("Device on ds0 is not a DS18x20 family device.");
return;
}//end of Switch
}//end of if bWhich==0

if (bWhich==1)
{
if ( !ds1.search(addr1))
{
  Serial.println("No more addresses on ds1.");
  Serial.println();
  ds1.reset_search();
  delay(250);
  return;
}

//Report the "ID" of the chip found...
Serial.print("ROM =");
for ( i = 0; i < 8; i++)
{
  Serial.write(' ');
  Serial.print(addr1[i], HEX);
}

if (OneWire::crc8(addr1, 7) != addr1[7])
{
  Serial.println("CRC is not valid for ds1!");
  return;
}
Serial.println();

// the first ROM byte indicates which chip
Serial.print("Chip on ds1: ");
switch (addr1[0])
{
case 0x10:
Serial.println("  Chip = DS18S20");  // or old DS1820
type_s1 = 1;
break;
case 0x28:
Serial.println("  Chip = DS18B20");
type_s1 = 0;
break;
case 0x22:
Serial.println("  Chip = DS1822");
type_s1 = 0;
break;
default:
Serial.println("Device on ds1 is not a DS18x20 family device.");
return;
}//end of Switch
}//end of if bWhich==1
}//End of "OneWireCheckChipAndInitVars()"


void OneWireAskThatReadingBeStarted(byte bWhich)
{
  //Ask chip to do tture conversion. Note: You must not reset 1-Wire MicroLan, nor ask
  //the chip for the conversion too soon after asking chip to DO conversion.
  //1 second is a reasonably safe "long enough". 2 seconds even better.

if (bWhich==0)
{
  ds0.reset();
  ds0.select(addr0);
  ds0.write(0x44, 1);        // start conversion, with parasite power on at the end
}

if (bWhich==1)
{
  ds1.reset();
  ds1.select(addr1);
  ds1.write(0x44, 1);        // start conversion, with parasite power on at the end
}
}//end of "OneWireAskThatReadingBeStarted()"

void OneWireFetchReadingThatWasTaken(byte bWhich)
//Note: This must not be called too quickly after
{
if (bWhich==0)
{
present=ds0.reset();
ds0.select(addr0);
ds0.write(0xBE);         // Read Scratchpad

/*All of the following is Good Stuff if you are
debugging some problem, but essentials are provided
for in more appropriate places for "day to day" needs...
Serial.print("  Data = ");
Serial.print(present, HEX);
Serial.print(" ");
*/

for ( i = 0; i < 9; i++) {           // we need 9 bytes
data[i] = ds0.read();

/*Good Stuff if you are debugging..
Serial.print(data[i], HEX);
Serial.print(" ");
*/
};// end of "for..."

/*CRC SHOULD BE **USED** to validate data received.
 *************************************************
Serial.print(" CRC=");
Serial.print(OneWire::crc8(data, 8), HEX);
Serial.println();
*/

//Convert bytes from chip to a tture in deg C....
int16_t raw = (data[1] << 8) | data[0];
if (type_s0) { //type_s was set earlier to reflect which of
  //several 1-Wire tture chips we are dealing with, so that
  //the sums peculiar to that chip are used...
raw = raw << 3; // 9 bit resolution default
if (data[7] == 0x10) {
// "count remain" gives full 12 bit resolution
raw = (raw & 0xFFF0) + 12 - data[6];
}
} else {
byte cfg = (data[4] & 0x60);
// at lower res, the low bits are undefined, so let's zero them
if (cfg == 0x00) raw = raw & ~7;  // 9 bit resolution, 93.75 ms
else if (cfg == 0x20) raw = raw & ~3; // 10 bit res, 187.5 ms
else if (cfg == 0x40) raw = raw & ~1; // 11 bit res, 375 ms
//// default is 12 bit resolution, 750 ms conversion time
}
celsius0 = (float)raw / 16.0;
}//end of if (bWhich==0)

if (bWhich==1)
{
present=ds1.reset();
ds1.select(addr1);
ds1.write(0xBE);         // Read Scratchpad

/*All of the following is Good Stuff if you are
debugging some problem, but essentials are provided
for in more appropriate places for "day to day" needs...
Serial.print("  Data = ");
Serial.print(present, HEX);
Serial.print(" ");
*/

for ( i = 0; i < 9; i++) {           // we need 9 bytes
data[i] = ds1.read();

/*Good Stuff if you are debugging..
Serial.print(data[i], HEX);
Serial.print(" ");
*/
};// end of "for..."

/*CRC SHOULD BE **USED** to validate data received.
 *************************************************
Serial.print(" CRC=");
Serial.print(OneWire::crc8(data, 8), HEX);
Serial.println();
*/

//Convert bytes from chip to a tture in deg C....
int16_t raw = (data[1] << 8) | data[0];
if (type_s1) { //type_s was set earlier to reflect which of
  //several 1-Wire tture chips we are dealing with, so that
  //the sums peculiar to that chip are used...
raw = raw << 3; // 9 bit resolution default
if (data[7] == 0x10) {
// "count remain" gives full 12 bit resolution
raw = (raw & 0xFFF0) + 12 - data[6];
}
} else {
byte cfg = (data[4] & 0x60);
// at lower res, the low bits are undefined, so let's zero them
if (cfg == 0x00) raw = raw & ~7;  // 9 bit resolution, 93.75 ms
else if (cfg == 0x20) raw = raw & ~3; // 10 bit res, 187.5 ms
else if (cfg == 0x40) raw = raw & ~1; // 11 bit res, 375 ms
//// default is 12 bit resolution, 750 ms conversion time
}
celsius1 = (float)raw / 16.0;
}//end of if (bWhich==1)

}//end of "OneWireFetchReadingThatWasTaken()"




void OneWireProcessCelsiusToOther(byte bWhich)
//The 1-wire chips all "speak" celsius. If, as I do, you
//want the temperature expressed differently, you start from
//the celsius, and generate what you want.
//So I've "hived off" the conversions which are less to do with
//talking to the 1-Wire chip, and more to do with the programmer's
//wants.

//There are "better" ways to do this, but this "will do" for my
//available time and energy: It makes a lot of "improper" use
//of globals. Before this routine can be usefully called, there
//must be a value in "celsius".

{
if (bWhich==0)
{
  fahrenheit0 = celsius0 * 1.8 + 32.0;

celsiusX10_0=celsius0*10;
//For "stuffing" first tture into web page....
  cSTture0='+';
  if (celsiusX10_0<0)
  {
    celsiusX10_0=celsiusX10_0*-1;
    cSTture0='-';
  }
  iTmp=celsiusX10_0;
  iTmp=iTmp%1000;//Just in case tture was somehow >99.9 deg, i.e. 999 deci-degrees
  //for debug Serial.println(iTmp);
  cTturDC0Huns=(iTmp/100)+48;//Hundreds(x10) character, temperature, in Celcuis DECIdecgrees... i.e. 12.4 degC is 124 deg
  //for debug Serial.println(cTturDC0Huns);
  iTmp=iTmp%100;
  cTturDC0Tens=(iTmp/10)+48;
  iTmp=iTmp%10;
  cTturDC0Units=(iTmp%10)+48;
}//if (bWhich==0)

if (bWhich==1)
{
  fahrenheit1 = celsius1 * 1.8 + 32.0;

celsiusX10_1=celsius1*10;
//For "stuffing" first tture into web page....
  cSTture1='+';
  if (celsiusX10_1<0)
  {
    celsiusX10_1=celsiusX10_1*-1;
    cSTture1='-';
  }
  iTmp=celsiusX10_1;
  iTmp=iTmp%1000;//Just in case tture was somehow >99.9 deg, i.e. 999 deci-degrees
  //for debug Serial.println(iTmp);
  cTturDC1Huns=(iTmp/100)+48;//Hundreds(x10) character, temperature, in Celcuis DECIdecgrees... i.e. 12.4 degC is 124 deg
  //for debug Serial.println(cTturDC1Huns);
  iTmp=iTmp%100;
  cTturDC1Tens=(iTmp/10)+48;
  iTmp=iTmp%10;
  cTturDC1Units=(iTmp%10)+48;
}//if (bWhich==0)

}//end of OneWireProcessCelsiusToOther()
